
:- ensure_loaded( library(lib) ).
% :- ensure_loaded( '/home/nicos/pl/packs/src/os_sub/prolog/os_sub' ).

:- lib(os_lib).
:- lib(by_unix).

:- debug(std_repo).
:- debug(by_unix).

:- ensure_loaded(bio_db_build_aliases), bio_db_build_aliases( [] ).
:- lib(stoics_lib:portray_clauses/2).
:- ensure_loaded( '../src/bio_db_repo_info.pl' ).

std_repo :-
    absolute_file_name( bio_db_build_downloads('.'), DnDir ),
    debug( std_repo, start, true ),
    os_path( BioDb, dnloads, DnDir ),
    write( bio_db(BioDb) ), nl,
    os_path( Work, BioDbDir, BioDb ),
    write( work(Work)-bio_db_dir(BioDbDir) ), nl,
    atomic_list_concat( [bio_db,Date], '-', BioDbDir ),
    write( date(Date) ), nl,
    atomic_list_concat( [bio_db_repo,Date], '-', RepoDir ),
    os_path( Work, RepoDir, Repo ),
    @ mkdir( Repo ),
    os_path( BioDb, data, BioDbData ),
    @ cp( -r, '--dereference', BioDbData, Repo ),
    os_path( Repo, data, Rata ),
    os_dir_dirs( Rata, RepoSubs ),
    debug( std_repo, 'Subs: ~w', [RepoSubs] ),
    maplist( std_repo_zip_cat(Rata), RepoSubs ),
    expand_file_name( '~/pl/packs/src/bio_db_repo', [PackRoot] ),
    os_path( PackRoot, doc, PackDoc ),
    os_path( PackRoot, prolog, PackPl ),
    @ cp( -r, PackDoc, Repo ),
    @ cp( -r, PackPl,  Repo ),
    os_path( Repo, 'prolog/bio_db_repo_version.pl', VersF ),
    atomic_list_concat( [YrA,MnA,DyA], '.', Date ),
    maplist( atom_number, [YrA,MnA,DyA], [Yr,Mn,Dy] ),
    portray_clauses( [bio_db_repo_version(Yr:Mn:Dy)], file(VersF) ),
    findall( Inf, bio_db_repo_info(Inf), [InfNm,InfTi|Infs] ),
    os_path( Repo, 'pack.pl', RepoPackF ),
    portray_clauses( [InfNm,InfTi,version(Date)|Infs], file(RepoPackF) ),
    os_ext( tar, RepoDir, TarF ),
    working_directory( Old, Work ),
    @ tar( cf, TarF, RepoDir ),
    @ gzip( TarF ),
    os_ext( gz, TarF, TarGzF ),
    os_ext( tgz, RepoDir, TgzF ),
    @ mv( TarGzF, TgzF ),
    expand_file_name( '~/web/sware/packs/bio_db_repo/', [WebD]  ),
    os_path( WebD, TgzF, WebTgzF ),
    ( exists_file(WebTgzF) -> 
        working_directory( _, Old ),
        throw( file_exists(WebTgzF) )
        ;
        true
    ),
    @ mv( TgzF, WebTgzF ),
    @ chmod( 'go+r', WebTgzF ),
    os_path( WebD, data, WebDataD ),
    @ rm( -f, WebDataD ),
    os_path( RepoDir, data, RepoDataDir ),
    atomic_list_concat( [data,Date], '-', DataDatedD ),
    os_path( WebD, DataDatedD, WebRepoDataD ),
    @ cp( -r, RepoDataDir, WebRepoDataD ),
    working_directory( _WorkA, WebD ),
    @ ln( -s, RepoDir, data ),
    @ mkvis( WebRepoDataD ),
    working_directory( _, Old ),
    @unison( 'stoicos.λύκος' ),
    debug( std_repo, stop, true ).

std_repo_zip_cat( Rata, Sub ) :-
    os_path( Rata, Sub, Path ),
    os_dir_dirs( Path, PathSubs ),
    maplist( std_repo_zip_dir(Path), PathSubs ).

std_repo_zip_dir( Rata, Sub ) :-
    os_path( Rata, Sub, Path ),
    os_dir_files( Path, Files ),
    % debug( std_repo, 'Dir: ~p Files: ~w', [Path,Files] ),
    working_directory( Old, Path ),
    maplist( bio_db_data_zip_file, Files ),
    working_directory( _, Old ).

bio_db_data_zip_file( File ) :-
    file_name_extension( _Stem, pl, File ),
    !,
    % os_path( Dir, File, Path ),
    debug( bio_db_data_zip, 'Doing: ~w', [File] ),
    file_name_extension( File, zip, ZipF ),
    @ zip( ZipF, File ),
    @ chmod( 'go+r', ZipF ),
    @ rm( -f, File ).
bio_db_data_zip_file( File ) :-
    debug( bio_db_data_zip, 'Skipping: ~w', [File] ).

